/*******************************************************************************
 * Copyright (C) 2017 Philipp Spohr
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
package de.hhu.ba.yoshikoWrapper.core;

import java.util.Properties;

import org.cytoscape.application.swing.CytoPanel;
import org.cytoscape.application.swing.CytoPanelComponent;
import org.cytoscape.application.swing.CytoPanelName;
import org.cytoscape.application.swing.CytoPanelState;
import org.cytoscape.model.CyColumn;
import org.cytoscape.model.CyNetwork;
import org.cytoscape.model.CyNode;
import org.cytoscape.work.AbstractTask;
import org.cytoscape.work.TaskMonitor;
import org.slf4j.Logger;

import de.hhu.ba.yoshikoWrapper.graphModel.YoshikoCluster;
import de.hhu.ba.yoshikoWrapper.graphModel.YoshikoResult;
import de.hhu.ba.yoshikoWrapper.graphModel.YoshikoSolution;
import de.hhu.ba.yoshikoWrapper.gui.ResultPanel;
import de.hhu.ba.yoshikoWrapper.logging.YoshikoLogger;
import de.hhu.ba.yoshikoWrapper.swig.ClusterEditingSolutions;
import de.hhu.ba.yoshikoWrapper.swig.CoreAlgorithm;
import de.hhu.ba.yoshikoWrapper.swig.IntVector;
import de.hhu.ba.yoshikoWrapper.swig.LibraryInput;
import de.hhu.ba.yoshikoWrapper.swig.LibraryInterface;


public class AlgorithmTask extends AbstractTask {
	
	
	//Symbolic links
	private static Logger logger = YoshikoLogger.getInstance().getLogger();

	//Parameters
	private final double insertionCostDefault;
	private final double deletionCostDefault;
	private final CyColumn permanentColumn;
	private final CyColumn forbiddenColumn;
	private CyColumn weightColumn;
	private int timeLimit;
	private CyNetwork net;
	private String bitMaskRules;
	private double multiplicativeFactor;
	private boolean separatePartitionCuts;
	private boolean separateTriangles;
	private boolean useHeuristic;
	private int numberOfSolutions;
	
	//Temps, need to be freed in C++
	private LibraryInput input;
	private ClusterEditingSolutions result;
	
	private CoreAlgorithm ca;
	private CallbackHandler ch;

	public AlgorithmTask(
			CyNetwork net,
			int timeLimit,
			CyColumn weightColumn,
			CyColumn permanentColumn,
			CyColumn forbiddenColumn,
			double insertionCostDefault,
			double deletionCostDefault,
			String bitMaskRules,
			double multiplicativeFactor,
			boolean separatePartitionCuts,
			boolean separateTriangles,
			boolean useHeuristic,
			int numberOfSolutions
			) 
	{
		this.net = net;
		this.timeLimit = timeLimit;
		this.weightColumn = weightColumn;
		this.permanentColumn = permanentColumn;
		this.forbiddenColumn = forbiddenColumn;
		this.insertionCostDefault = insertionCostDefault;
		this.deletionCostDefault = deletionCostDefault;
		this.bitMaskRules = bitMaskRules;
		this.multiplicativeFactor = multiplicativeFactor;
		this.separatePartitionCuts = separatePartitionCuts;
		this.separateTriangles = separateTriangles;
		this.useHeuristic = useHeuristic;
		this.numberOfSolutions = numberOfSolutions;
	}

	@Override
	public void run(TaskMonitor taskMonitor) throws Exception {


		taskMonitor.setTitle(LocalizationManager.get("yoshTask"));
		taskMonitor.setProgress(0.0);
		//Get current network
		if (net == null) {
			logger.warn("CoreAlgorithm called on a net that is NULL!");
			return;
		}
		
		//Set time limit
		LibraryInterface.setTimeLimit(timeLimit);

		//Create a node map to identify nodes and eges in the solution
		NodeMap nodeMap  = new NodeMap(net);
		taskMonitor.setProgress(0.1);

		//Generate an input instance from the network
		input = NetworkParser.parseNetwork(
				net,
				nodeMap,
				weightColumn,
				permanentColumn, 
				forbiddenColumn,
				deletionCostDefault
				);
		taskMonitor.setProgress(0.2);

		//Set the default value for insertion cost
		input.setDefaultInsertionCost(insertionCostDefault);
		
		//Call Yoshiko <<< Algorithm is performed here
		ca = LibraryInterface.getRun(input, 
				numberOfSolutions,
				bitMaskRules,
				multiplicativeFactor, 
				separatePartitionCuts,
				separateTriangles,
				useHeuristic
		);
		result  = ca.run();
		
		taskMonitor.setProgress(0.9);
		
		long numberOfSolutions = result.getNumberOfSolutions();
		taskMonitor.setStatusMessage("Found: "+numberOfSolutions+" solutions!"); //TODO localize
				
		YoshikoResult yoshikoResult = new YoshikoResult();
		
		yoshikoResult.editingCost = result.getTotalCost();
		
		//Loop over (multiple) solutions
		for (long i=0;i<numberOfSolutions;i++) {
			
			taskMonitor.setStatusMessage("Processing solution "+(i+1)+" of "+numberOfSolutions);
			YoshikoSolution solution = new YoshikoSolution(net);
			String columnName = "YOSHIKO_SOLUTION_"+(i+1);
			
			net.getDefaultNodeTable().deleteColumn(columnName);
			net.getDefaultNodeTable().createColumn(columnName, String.class, false);
			
			//Fetch number of clusters in the solution
			long numberOfClusters = result.getNumberOfClusters(i);
			
			//Loop over cluster
			for (long k=0;k<numberOfClusters;k++) {
				//Create java instance
				YoshikoCluster cluster = new YoshikoCluster(k,net);

				
				taskMonitor.setStatusMessage("Processing cluster "+(k+1)+" of "+numberOfClusters);
				
				IntVector clusterVector = result.getCluster(i, k);
				
				long sizeOfCluster = clusterVector.size();
				for (int l=0;l<sizeOfCluster;l++) { //Unsafe mismatch int long
					taskMonitor.setStatusMessage("Processing entry "+(l+1)+ " of "+sizeOfCluster);
					int nodeID = clusterVector.get(l);
					CyNode node = nodeMap.indexOf(nodeID); //<<< Another int/long conversion
					cluster.addNode(node);
					
					net.getRow(node).set(columnName, ""+(k+1)); //Add Cluster ID in table (Remove in final version?)
				}
				solution.cluster.add(cluster);
			}
			yoshikoResult.solutions.add(solution);
		}
		
		
		//Provide some useful info
		if (result.isTimedOut()) {
			throw new Exception(LocalizationManager.get("timedOutMessage"));
		}

		//At this point the C++ objects can be freed, in case that the algorithm terminates beforehand they need to be cleaned-up elsewhere
		ca.delete();
		input.delete();
		result.delete();
		
		//Generate solutionsPanel
		ResultPanel solutionsPanel = new ResultPanel(yoshikoResult);

		//Show solution panel
		CyCore.registrar.registerService(solutionsPanel,CytoPanelComponent.class, new Properties());
		//Focus solution panel
		CytoPanel eastPanel = CyCore.swing.getCytoPanel(CytoPanelName.EAST);
		eastPanel.setSelectedIndex(eastPanel.indexOfComponent(solutionsPanel));
		//Show (might be invisible)
		eastPanel.setState(CytoPanelState.DOCK);

	}
	
	@Override
	public void cancel() {
		//Free C++ resources
		if (ca != null) {
			System.out.println("Deleting CoreAlgorithm instance!");
			ca.cancel();
			ca.delete();
		}
		if (input != null) {
			input.delete();
		}
		if (result != null) {
			result.delete();
		}
		super.cancel();
	}
}
