/*******************************************************************************
 * Copyright (C) 2017 Philipp Spohr
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 ******************************************************************************/
package de.hhu.ba.yoshikoWrapper.gui;

import static javax.swing.GroupLayout.DEFAULT_SIZE;
import static javax.swing.GroupLayout.PREFERRED_SIZE;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.GroupLayout;
import javax.swing.GroupLayout.Alignment;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.cytoscape.application.swing.CytoPanelComponent;
import org.cytoscape.application.swing.CytoPanelName;
import org.cytoscape.work.AbstractTask;
import org.cytoscape.work.TaskIterator;

import de.hhu.ba.yoshikoWrapper.core.AlgorithmTask;
import de.hhu.ba.yoshikoWrapper.core.CyCore;
import de.hhu.ba.yoshikoWrapper.core.GraphicsLoader;
import de.hhu.ba.yoshikoWrapper.core.LocalizationManager;
import de.hhu.ba.yoshikoWrapper.core.YoshikoLoader;

/**This class describes the Swing Panel that the user interacts with in cytoscape
 *
 */
@SuppressWarnings("serial")
public class MainPanel extends JPanel implements CytoPanelComponent {

	//SWING COMPONENTS
	
	private final YoshikoHeader header;
	
	private final JCheckBox showAdvancedOptions;
	private final ArrayList<JComponent> advancedOptions;

	private final LanguageSwitcherPanel langPanel;
	private final LibraryPanel libraryPanel;
	private final EditCostPanel ecPanel;
	private final ReductionRulesChooser reductionRulesChooser;
	private final OperationModePanel opModePanel;	
	
	/**
	 * Main constructor, creates a new Panel and initializes subcomponents
	 */
	public MainPanel() {
				
		//SWING INIT

		
		//Initialize Swing components
		header = new YoshikoHeader();
		
		showAdvancedOptions = new JCheckBox(LocalizationManager.get("showAdvanced"));
		showAdvancedOptions.addActionListener(toggleAdvancedOptionsListener);
		
		langPanel = new LanguageSwitcherPanel();
		
		libraryPanel = new LibraryPanel();
		//If no library is loaded yet the obvious panel should be showing up
		if (!YoshikoLoader.isLibraryLoaded()) {
			libraryPanel.setCollapsed(false);
		}
		
		ecPanel = new EditCostPanel();
		
		opModePanel = new OperationModePanel();
		
		reductionRulesChooser = new ReductionRulesChooser();
		
		JButton runButton = new JButton(LocalizationManager.get("run"));
		runButton.addActionListener(buttonListener);

		//Add components to main panel

		
		SwingUtil.addAll(this,
				header,
				showAdvancedOptions,
				langPanel,
				libraryPanel,
				ecPanel,
				reductionRulesChooser,
				opModePanel,
				runButton
		);
		
		//Manage all advanced components separately to enable toggling
		advancedOptions = new ArrayList<JComponent>();
		advancedOptions.addAll(
				Arrays.asList(
						opModePanel,
						reductionRulesChooser
						)
		);
		
		showAdvancedOptions(false);
		
		//Layout
		GroupLayout layout = new GroupLayout(this);
		layout.setAutoCreateGaps(true);
		layout.setAutoCreateContainerGaps(true);
		
		layout.setHorizontalGroup(layout.createParallelGroup(Alignment.LEADING,true)
				.addGap(30)
				.addComponent(header,DEFAULT_SIZE, DEFAULT_SIZE, Short.MAX_VALUE)
				.addComponent(showAdvancedOptions,DEFAULT_SIZE, DEFAULT_SIZE, Short.MAX_VALUE)
				.addComponent(langPanel,DEFAULT_SIZE, DEFAULT_SIZE, Short.MAX_VALUE)
				.addComponent(libraryPanel,PREFERRED_SIZE, PREFERRED_SIZE, Short.MAX_VALUE)
				.addComponent(ecPanel,DEFAULT_SIZE, DEFAULT_SIZE, Short.MAX_VALUE)
				.addComponent(reductionRulesChooser,DEFAULT_SIZE, DEFAULT_SIZE, Short.MAX_VALUE)
				.addComponent(opModePanel,DEFAULT_SIZE, DEFAULT_SIZE, Short.MAX_VALUE)
				.addComponent(runButton,DEFAULT_SIZE, DEFAULT_SIZE, Short.MAX_VALUE)
		);
		
		layout.setVerticalGroup(layout.createSequentialGroup()
				.addComponent(header,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(showAdvancedOptions,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(langPanel,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(libraryPanel,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(ecPanel,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(reductionRulesChooser,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(opModePanel,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)
				.addComponent(runButton,PREFERRED_SIZE,PREFERRED_SIZE,PREFERRED_SIZE)	
		);
		
		this.setLayout(layout);
		
		
	}
	
	private ActionListener toggleAdvancedOptionsListener = new ActionListener() {

		@Override
		public void actionPerformed(ActionEvent e) {
			showAdvancedOptions(showAdvancedOptions.isSelected());
		}
		
	};
	
	private void showAdvancedOptions(boolean show) {
		for (JComponent j : advancedOptions) {
			j.setVisible(show);
		}
		setPreferredSize(getPreferredSize()); // <<< Swing at its finest
		revalidate();
	}

	/**
	 * ButtonListener for the "Run" Button
	 * Handles calling the algorithm and fetching/passing the arguments from swing components
	 */
	private ActionListener buttonListener = new ActionListener() {

		private final TaskIterator iterator = new TaskIterator();
		@Override
		public void actionPerformed(ActionEvent e) {
			if (YoshikoLoader.isLibraryLoaded()){
				AbstractTask yoshiko = new AlgorithmTask(
					CyCore.cy.getCurrentNetwork(),
					opModePanel.getTimeLimit(),
					ecPanel.getWeightColumn(),
					ecPanel.getPermanentColumn(),
					ecPanel.getForbiddenColumn(),
					ecPanel.getDefaultInsertionCost(),
					ecPanel.getDefaultDeletionCost(),
					reductionRulesChooser.getBitMask(),
					reductionRulesChooser.getMultFactor(),
					opModePanel.useTriangleCuts(),
					opModePanel.usePartitionCuts(), 
					opModePanel.useHeuristic(),
					opModePanel.getSolCount()
				);
				/**
				 * Don't queue multiple runs at once because that will get messy
				 */
				if (!iterator.hasNext()) {
					iterator.append(yoshiko);
					CyCore.dialogTaskManager.execute(iterator);
				}
			}
			else {
				JOptionPane.showMessageDialog(	
					null,
					LocalizationManager.get("noLibTitle"),
					LocalizationManager.get("noLibMessage"),
					JOptionPane.ERROR_MESSAGE
				);
			}
		}
	};
	

	
	//GETTER / SETTER

	public Component getComponent() {
		return this;
	}

	/* (non-Javadoc)
	 * @see org.cytoscape.application.swing.CytoPanelComponent#getCytoPanelName()
	 */
	public CytoPanelName getCytoPanelName() {
		//By convention most plugins that provide a "toolbox"-like interface use the WEST orientation
		return CytoPanelName.WEST;
	}

	public String getTitle() {
		//TODO: Be creative I guess
		return "Yoshiko";
	}
	
	public ColumnMapper getColumnMapper() {
		return ecPanel.getColumnMapper();
	}
	
	public Icon getIcon() {
		return GraphicsLoader.getLogo(16);
	}


}
